#!/usr/bin/python
import gtk, gtk.glade, pango, gobject
from abersimeditor import Editor
from abersimlisting import Listing
import sys, os, select


class AberSim(gtk.Window):

        def __init__(self):
		self.mainGlade = gtk.glade.XML("abersim.glade", "winMain", None)
		
		signals = {
			"on_winMain_destroy" : self.quit,
			"on_quit_activate" : self.quit,
			"on_drawing_expose_event" : self.expose_event,
			"on_new_activate" : self.new,
			"on_open_activate" : self.open,
			"on_about_activate" : self.about,
		}
	
		self.mainGlade.signal_autoconnect(signals)
		
		self.lblStatus = self.mainGlade.get_widget("lblStatus")
		self.lblAddr = self.mainGlade.get_widget("lblAddr")
		self.lblInst = self.mainGlade.get_widget("lblInst")
		self.lblTicks = self.mainGlade.get_widget("lblTicks")
		self.lblPC = self.mainGlade.get_widget("lblPC")
		self.lblAB = self.mainGlade.get_widget("lblAB")
		self.lblX = self.mainGlade.get_widget("lblX")
		self.lblY = self.mainGlade.get_widget("lblY")
		self.lblSP = self.mainGlade.get_widget("lblSP")
		self.lblCCR = self.mainGlade.get_widget("lblCCR")
		self.lblPortB = self.mainGlade.get_widget("lblPortB")
		self.lblPortC = self.mainGlade.get_widget("lblPortC")
		self.spnSteps = self.mainGlade.get_widget("spnSteps")
		self.drawing = self.mainGlade.get_widget("drawing")

		self.black_gc = gtk.gdk.GC(self.drawing.window)
		self.black_gc.set_rgb_fg_color(gtk.gdk.color_parse('black'))
		self.white_gc = gtk.gdk.GC(self.drawing.window)
		self.white_gc.set_rgb_fg_color(gtk.gdk.color_parse('white'))

		self.listing = None
		self.running = False
		self.executing = False

	def quit(self, widget):
		gtk.main_quit()

	def new(self, widget):
		editor = Editor(self)
		
	def open(self, widget):
		self.openGlade = gtk.glade.XML("abersim.glade", "dlgOpen", None)

		signals = {
			"on_btnCancel_clicked" : self.closeopener,
			"on_btnOpen_clicked" : self.openfile,
		}

		self.openGlade.signal_autoconnect(signals)

	def closeopener(self, widget):
		dlg = self.openGlade.get_widget("dlgOpen")
		dlg.destroy()

	def openfile(self, widget):
		dlg = self.openGlade.get_widget("dlgOpen")
		filename = dlg.get_filename()
		editor = Editor(self, filename)
		self.closeopener(widget)

	def about(self, widget):
		gtk.glade.XML("abersim.glade", "dlgAbout", None)

	def expose_event(self, widget, *args):
		event = args[0]
		top, left, width, height = event.area
		self.update(top, left, width, height)

	def update(self, top, left, width, height):
		self.drawing.window.draw_rectangle(self.white_gc,  1, top, left, width, height)

	def execute(self, asmfile):
		self.running = False
		win = self.mainGlade.get_widget("winMain")
		win.present()
		self.asmfile = asmfile
		self.stdin, self.stdout, self.stderr = os.popen3("sim6811 " + asmfile + ".s19")
		if self.listing:
			self.listing.close()
		self.listing = Listing(self, asmfile + ".lst")
		self.running = False
		self.toggle()
		#Don't want to run if we've already been executing something or timeouts will stack up
		if not self.executing:
			self.output()
		self.executing = True
		
	def run(self):
		if self.running:
			self.step()
			gobject.timeout_add(int(1000 / self.spnSteps.get_value()), self.run)

	#This needs rewriting to separate output from input
	def step(self, widget = None):
		sout, sin, serr = select.select([self.stdout], [self.stdin] , [self.stderr])
		if sin:
			self.stdin.write("md 1003 1\nmd 1004 1\ns\n")
			self.stdin.flush()
																	
	def output(self):
		sout, sin, serr = select.select([self.stdout], [self.stdin] , [self.stderr] )

		if sout:
			out = self.stdout.readline()
			out = out.replace("\n", "")
			out = out.replace(">", "")

			if out[:3] == "PC=":
				#Parse output
				pc, ab, x, y, sp, ccr = out.split(" ", 6)
				ccr, ticks = ccr.split("\t", 1)
				ticks = ticks.replace("[", "")
				ticks = ticks.replace("]", "")
				pc = pc.split("=", 1)[1]
				ab = ab.split("=", 1)[1]
				x = x.split("=", 1)[1]
				y = y.split("=", 1)[1]
				sp = sp.split("=", 1)[1]
				ccr = ccr.split("=", 1)[1]
			
				#Get second line of output
				out2 = self.stdout.readline().replace("\n", "")
				address, instr= out2.split("\t", 2)

				self.listing.set_line(address)
			
				self.lblAddr.set_text(address)
				self.lblInst.set_text(instr)
				self.lblTicks.set_text(ticks)
				self.lblPC.set_text(pc)
				self.lblAB.set_text(ab)
				self.lblX.set_text(x)
				self.lblY.set_text(y)
				self.lblSP.set_text(sp)
				self.lblCCR.set_text(ccr)

			elif out[:4] == "1004":		#Port B
				portb = out.split("\t")[1]
				portb = portb[:2]
				self.lblPortB.set_text(portb)

			elif out[:4] == "1003":		#Port
				portc = out.split("\t")[1]
				portc = portc[:2]
				self.lblPortC.set_text(portc)

		if serr:
			print self.stderr.readlines()
		
		if self.running:
			timing = int(250 / self.spnSteps.get_value()) #Must be 4 times faster than input, since we're receiving 4 pieces of data per cycle
			gobject.timeout_add(timing, self.output)
		else:	#Need to keep up output for stepping
			gobject.timeout_add(250, self.output)
	
	def toggle(self, widget = None):
		self.running = not self.running
		self.run()
		if self.running:
			self.lblStatus.set_text("Running...")
		else:
			self.lblStatus.set_text("Paused.")

prog = AberSim()
#Open file from commandline
if len(sys.argv) > 1:
	dir, file = os.path.split(sys.argv[1])
	if not dir:
		dir = os.getcwd()
	file = os.path.join(dir, file)
	editor = Editor(prog, file)
gtk.main()
