#!/usr/bin/python
import gtk, gtk.glade, pango, gobject
from gtksourceview import *
import os, select


class Editor(gtk.Window):

	def __init__(self, parent, filename = None):
		self.sim = parent
		self.mainGlade = gtk.glade.XML("abersim.glade", "winEditor", None)

		signals = {
			"on_btnExec_clicked" : self.execute,
			"on_btnOpen_clicked" : self.open,
			"on_btnSave_clicked" : self.save,
			"on_btnSaveAs_clicked" : self.saveas,
		}

		self.mainGlade.signal_autoconnect(signals)
		
		#Setup source viewer
		scrEdit = self.mainGlade.get_widget("scrEdit")
		buffer = SourceBuffer()
		buffer.set_highlight(True)
		buffer.set_check_brackets(False)
		self.filename = filename
		if filename:
			file = open(self.filename, "r")
			buffer.set_text(file.read())
			file.close()
		manager = SourceLanguagesManager()
		language = manager.get_language_from_mime_type("text/asm6811")
		buffer.set_language(language)
		self.srcview = SourceView(buffer)
		self.srcview.set_tabs_width(16)
		scrEdit.add(self.srcview)
		self.srcview.show()

	def execute(self, widget = None):
		self.save()
		if self.filename:
			dir, file = os.path.split(self.filename)
			currdir = os.getcwd()
			os.chdir(dir)
			asmfile = file.rsplit(".", 1)[0]
			#Need to display the assembler's output nicely at some point
			asm = os.popen("as11 " + file + " -l > " + asmfile + ".lst")
			os.chdir(currdir)
			self.sim.execute(os.path.join(dir,asmfile))

	def open(self, widget = None):
		self.openGlade = gtk.glade.XML("abersim.glade", "dlgOpen", None)

		signals = {
			"on_btnCancel_clicked" : self.closeopener,
			"on_btnOpen_clicked" : self.openfile,
		}

		self.openGlade.signal_autoconnect(signals)

	def closeopener(self, widget = None):
		dlg = self.openGlade.get_widget("dlgOpen")
		dlg.destroy()
	
	def openfile(self, widget = None):
		dlg = self.openGlade.get_widget("dlgOpen")
		filename = dlg.get_filename()
		editor = Editor(self.sim, filename)
		self.closeopener()

	def save(self, widget = None):
		if self.filename:
			file = open(self.filename, "w")
			buffer = self.srcview.get_buffer()
			file.write(buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter()))
			file.close()
		else:
			self.saveas()

	def saveas(self, widget = None):
		self.saveGlade = gtk.glade.XML("abersim.glade", "dlgSave", None)

		signals = {
			"on_btnCancel_clicked" : self.closesaver,
			"on_btnOpen_clicked" : self.savefile,
		}

		self.saveGlade.signal_autoconnect(signals)

	def closesaver(self, widget = None):
		dlg = self.saveGlade.get_widget("dlgSave")
		dlg.destroy()

	def savefile(self, widget = None):
		dlg = self.saveGlade.get_widget("dlgSave")
		filename = dlg.get_filename()
		self.filename = filename
		self.closesaver()
		self.save()
