#!/usr/bin/python
import gtk, gtk.glade, pango, gobject
from gtksourceview import *
import os, select, codecs


class Listing(gtk.Window):

	def __init__(self, parent, filename = None):
		self.sim = parent
		self.mainGlade = gtk.glade.XML("abersim.glade", "winListing", None)

		signals = {
			"on_btnPause_toggled" : self.toggle,
			"on_btnStep_clicked" : self.sim.step,
		}

		self.mainGlade.signal_autoconnect(signals)
		
		#Setup source viewer
		scrList = self.mainGlade.get_widget("scrList")
		buffer = SourceBuffer()
		buffer.set_highlight(True)
		buffer.set_check_brackets(False)
		self.filename = filename
		if filename:
			file = open(self.filename, "r")
			buffer = self.load(buffer, file.readlines())
			file.close()
		manager = SourceLanguagesManager()
		language = manager.get_language_from_mime_type("text/6811listing")
		buffer.set_language(language)
		self.srcview = SourceView(buffer)
		self.srcview.set_show_line_numbers(True)
		self.srcview.set_show_line_markers(True)
		self.srcview.set_editable(False)
		self.srcview.set_cursor_visible(False)
		self.srcview.set_sensitive(False)
		font_desc = pango.FontDescription('monospace')
		self.srcview.modify_font(font_desc)
		scrList.add(self.srcview)
		self.srcview.show()

	def load(self, buffer, lines):
		buffer.begin_not_undoable_action()
		text = ""
		self.lineaddress = { "0000" : 1} #Keep track of what memory is on what line
		self.linelength = [] #Length of each line for highlighting
		for line in lines:
			if line[:6] == "output": #Not part of listing
				continue
			lineno = line[:5]
			
			line = self.fixasm11(line)

			line = line[5:] #Strip line numbers
			address = line[:4]
			self.lineaddress[address] = int(lineno) - 1 #TextBuffer lines are offset from sourceview's side numbering
			self.linelength.append(len(line) - 1)
			text += line
		text = text[:-1] #Remove final new line
		buffer.set_text(text)
		buffer.end_not_undoable_action()
		return buffer

	def toggle(self, widget = None):
		self.sim.toggle()
		btnPause = self.mainGlade.get_widget("btnPause")
		btnStep = self.mainGlade.get_widget("btnStep")
		btnBreak = self.mainGlade.get_widget("btnBreak")
		if btnPause.get_active():
			btnStep.set_sensitive(True)
			btnBreak.set_sensitive(True)
		else:
			btnStep.set_sensitive(False)
			btnBreak.set_sensitive(False)

	def close(self):
		win = self.mainGlade.get_widget("winListing")
		win.destroy()

	def set_line(self, address):
		lineno = self.lineaddress[address]
		buffer = self.srcview.get_buffer()
		lineiter = buffer.get_iter_at_line(lineno)
		endlineiter = buffer.get_iter_at_line_offset(lineno, self.linelength[lineno])
		buffer.place_cursor(lineiter)
		self.srcview.scroll_to_iter(lineiter, 0)
		buffer.select_range(lineiter, endlineiter)

	def fixasm11(self, text):
		'''Fix the broken listing output from as11'''
		newtext = ""
		for char in text:
			if ord(char) == 0:
				break
			newtext += char

		return newtext
			
